@@ -207,7 +207,6 @@ static UINT audin_process_formats(AUDIN_PLUGIN* audin, AUDIN_CHANNEL_CALLBACK* c
 
 	Stream_Seek_UINT32(s); /* cbSizeFormatsPacket */
 
-	audin->format = NULL;
 	audio_formats_free(callback->formats, callback->formats_count);
 	callback->formats_count = 0;
 
@@ -285,7 +284,6 @@ out:
 
 	if (error != CHANNEL_RC_OK)
 	{
-		audin->format = NULL;
 		audio_formats_free(callback->formats, NumFormats);
 		callback->formats = NULL;
 	}
@@ -671,7 +669,7 @@ static UINT audin_on_new_channel_connection(IWTSListenerCallback* pListenerCallb
 	callback->plugin = listener_callback->plugin;
 	callback->channel_mgr = listener_callback->channel_mgr;
 	callback->channel = pChannel;
-	*ppCallback = &callback->iface;
+	*ppCallback = (IWTSVirtualChannelCallback*)callback;
 	return CHANNEL_RC_OK;
 }
 
