@@ -22,7 +22,6 @@
 #include <winpr/interlocked.h>
 
 #include "camera.h"
-#include "rdpecam-utils.h"
 
 #define TAG CHANNELS_TAG("rdpecam-device.client")
 
@@ -65,22 +64,14 @@ static BOOL ecam_dev_read_media_type(wStream* s, CAM_MEDIA_TYPE_DESCRIPTION* med
 {
 	WINPR_ASSERT(mediaType);
 
-	const uint8_t format = Stream_Get_UINT8(s);
-	if (!rdpecam_valid_CamMediaFormat(format))
-		return FALSE;
-
-	mediaType->Format = WINPR_ASSERTING_INT_CAST(CAM_MEDIA_FORMAT, format);
+	Stream_Read_UINT8(s, mediaType->Format);
 	Stream_Read_UINT32(s, mediaType->Width);
 	Stream_Read_UINT32(s, mediaType->Height);
 	Stream_Read_UINT32(s, mediaType->FrameRateNumerator);
 	Stream_Read_UINT32(s, mediaType->FrameRateDenominator);
 	Stream_Read_UINT32(s, mediaType->PixelAspectRatioNumerator);
 	Stream_Read_UINT32(s, mediaType->PixelAspectRatioDenominator);
-
-	const uint8_t flags = Stream_Get_UINT8(s);
-	if (!rdpecam_valid_MediaTypeDescriptionFlags(flags))
-		return FALSE;
-	mediaType->Flags = WINPR_ASSERTING_INT_CAST(CAM_MEDIA_TYPE_DESCRIPTION_FLAGS, flags);
+	Stream_Read_UINT8(s, mediaType->Flags);
 	return TRUE;
 }
 
@@ -88,7 +79,7 @@ static void ecam_dev_print_media_type(CAM_MEDIA_TYPE_DESCRIPTION* mediaType)
 {
 	WINPR_ASSERT(mediaType);
 
-	WLog_DBG(TAG, "Format: %u, width: %u, height: %u, fps: %u, flags: %u", mediaType->Format,
+	WLog_DBG(TAG, "Format: %d, width: %d, height: %d, fps: %d, flags: %d", mediaType->Format,
 	         mediaType->Width, mediaType->Height, mediaType->FrameRateNumerator, mediaType->Flags);
 }
 
@@ -130,7 +121,7 @@ static BOOL mediaSupportDrops(CAM_MEDIA_FORMAT format)
 	}
 }
 
-static UINT ecam_dev_send_pending(CameraDevice* dev, size_t streamIndex, CameraDeviceStream* stream)
+static UINT ecam_dev_send_pending(CameraDevice* dev, int streamIndex, CameraDeviceStream* stream)
 {
 	WINPR_ASSERT(dev);
 	WINPR_ASSERT(stream);
@@ -169,10 +160,11 @@ static UINT ecam_dev_send_pending(CameraDevice* dev, size_t streamIndex, CameraD
 	stream->samplesRequested--;
 	stream->haveSample = FALSE;
 
-	return ecam_dev_send_sample_response(dev, streamIndex, encodedSample, encodedSize);
+	return ecam_dev_send_sample_response(dev, WINPR_ASSERTING_INT_CAST(size_t, streamIndex),
+	                                     encodedSample, encodedSize);
 }
 
-static UINT ecam_dev_sample_captured_callback(CameraDevice* dev, size_t streamIndex,
+static UINT ecam_dev_sample_captured_callback(CameraDevice* dev, int streamIndex,
                                               const BYTE* sample, size_t size)
 {
 	WINPR_ASSERT(dev);
@@ -307,7 +299,7 @@ static UINT ecam_dev_process_start_streams_request(CameraDevice* dev,
 
 	if (streamIndex >= ECAM_DEVICE_MAX_STREAMS)
 	{
-		WLog_ERR(TAG, "Incorrect streamIndex %" PRIu8, streamIndex);
+		WLog_ERR(TAG, "Incorrect streamIndex %" PRIuz, streamIndex);
 		ecam_channel_send_error_response(dev->ecam, hchannel, CAM_ERROR_CODE_InvalidStreamNumber);
 		return ERROR_INVALID_INDEX;
 	}
@@ -376,8 +368,8 @@ static UINT ecam_dev_process_start_streams_request(CameraDevice* dev,
 		return ERROR_INVALID_DATA;
 	}
 
-	const CAM_ERROR_CODE error = dev->ihal->StartStream(dev->ihal, dev, streamIndex, &mediaType,
-	                                                    ecam_dev_sample_captured_callback);
+	UINT error = dev->ihal->StartStream(dev->ihal, dev, streamIndex, &mediaType,
+	                                    ecam_dev_sample_captured_callback);
 	if (error)
 	{
 		WLog_ERR(TAG, "StartStream failure");
@@ -665,7 +657,7 @@ static UINT ecam_dev_process_activate_device_request(CameraDevice* dev,
                                                      WINPR_ATTR_UNUSED wStream* s)
 {
 	WINPR_ASSERT(dev);
-	CAM_ERROR_CODE errorCode = CAM_ERROR_CODE_None;
+	UINT32 errorCode = 0;
 
 	if (dev->ihal->Activate(dev->ihal, dev->deviceId, &errorCode))
 		return ecam_channel_send_generic_msg(dev->ecam, hchannel, CAM_MSG_ID_SuccessResponse);
@@ -688,7 +680,7 @@ static UINT ecam_dev_process_deactivate_device_request(CameraDevice* dev,
 	for (size_t i = 0; i < ECAM_DEVICE_MAX_STREAMS; i++)
 		ecam_dev_stop_stream(dev, i);
 
-	CAM_ERROR_CODE errorCode = CAM_ERROR_CODE_None;
+	UINT32 errorCode = 0;
 	if (dev->ihal->Deactivate(dev->ihal, dev->deviceId, &errorCode))
 		return ecam_channel_send_generic_msg(dev->ecam, hchannel, CAM_MSG_ID_SuccessResponse);
 
@@ -720,7 +712,7 @@ static UINT ecam_dev_on_data_received(IWTSVirtualChannelCallback* pChannelCallba
 
 	Stream_Read_UINT8(data, version);
 	Stream_Read_UINT8(data, messageId);
-	WLog_DBG(TAG, "ChannelId=%" PRIu32 ", MessageId=0x%02" PRIx8 ", Version=%d",
+	WLog_DBG(TAG, "ChannelId=%d, MessageId=0x%02" PRIx8 ", Version=%d",
 	         hchannel->channel_mgr->GetChannelId(hchannel->channel), messageId, version);
 
 	switch (messageId)
@@ -798,9 +790,6 @@ static UINT ecam_dev_on_close(IWTSVirtualChannelCallback* pChannelCallback)
 
 	WLog_DBG(TAG, "entered");
 
-	for (size_t i = 0; i < ECAM_DEVICE_MAX_STREAMS; i++)
-		ecam_dev_stop_stream(dev, i);
-
 	/* make sure this channel is not used for sample responses */
 	for (size_t i = 0; i < ECAM_DEVICE_MAX_STREAMS; i++)
 		if (dev->streams[i].hSampleReqChannel == hchannel)
