@@ -71,21 +71,8 @@ extern "C"
 	typedef BOOL (*psPeerCapabilities)(freerdp_peer* peer);
 	typedef BOOL (*psPeerPostConnect)(freerdp_peer* peer);
 	typedef BOOL (*psPeerActivate)(freerdp_peer* peer);
-
-	/** @brief Callback after the initial RDP authentication (NLA) succeeded or anonymous tunnel was
-	 * established (RDP, TLS, ...)
-	 *
-	 *  @param peer A pointer to a peer context to work on
-	 *  @param identity A pointer to the identity of the peer
-	 *  @param automatic \b TRUE in case the connection is already authenticated, \b FALSE in case
-	 * of \b RDP, \b TLS or similar anonymous tunnels
-	 *
-	 *  @return \b TRUE if the connection is allowed, \b FALSE if denied. Defaults to \b TRUE if the
-	 * callback is unused.
-	 */
 	typedef BOOL (*psPeerLogon)(freerdp_peer* peer, const SEC_WINNT_AUTH_IDENTITY* identity,
 	                            BOOL automatic);
-
 	typedef BOOL (*psPeerSendServerRedirection)(freerdp_peer* peer,
 	                                            const rdpRedirection* redirection);
 	typedef BOOL (*psPeerAdjustMonitorsLayout)(freerdp_peer* peer);
@@ -140,39 +127,38 @@ extern "C"
 
 		ALIGN64 void* ContextExtra;
 		ALIGN64 size_t ContextSize;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerContextNew ContextNew;
+		ALIGN64 psPeerContextNew ContextNew;
 		ALIGN64 psPeerContextFree ContextFree;
 
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerInitialize Initialize;
+		ALIGN64 psPeerInitialize Initialize;
 #if defined(WITH_FREERDP_DEPRECATED)
 		WINPR_DEPRECATED_VAR("Use freerdp_peer::GetEventHandle instead",
-		                     WINPR_ATTR_NODISCARD ALIGN64 psPeerGetFileDescriptor
-		                         GetFileDescriptor;)
+		                     ALIGN64 psPeerGetFileDescriptor GetFileDescriptor;)
 #else
 	UINT64 reserved;
 #endif
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerGetEventHandle GetEventHandle;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerGetReceiveEventHandle GetReceiveEventHandle;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerCheckFileDescriptor CheckFileDescriptor;
+		ALIGN64 psPeerGetEventHandle GetEventHandle;
+		ALIGN64 psPeerGetReceiveEventHandle GetReceiveEventHandle;
+		ALIGN64 psPeerCheckFileDescriptor CheckFileDescriptor;
 		ALIGN64 psPeerClose Close;
 		ALIGN64 psPeerDisconnect Disconnect;
 
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerCapabilities Capabilities;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerPostConnect PostConnect;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerActivate Activate;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerLogon Logon;
+		ALIGN64 psPeerCapabilities Capabilities;
+		ALIGN64 psPeerPostConnect PostConnect;
+		ALIGN64 psPeerActivate Activate;
+		ALIGN64 psPeerLogon Logon;
 
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerSendServerRedirection SendServerRedirection;
+		ALIGN64 psPeerSendServerRedirection SendServerRedirection;
 
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerSendChannelData SendChannelData;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerReceiveChannelData ReceiveChannelData;
+		ALIGN64 psPeerSendChannelData SendChannelData;
+		ALIGN64 psPeerReceiveChannelData ReceiveChannelData;
 
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerVirtualChannelOpen VirtualChannelOpen;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerVirtualChannelClose VirtualChannelClose;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerVirtualChannelRead VirtualChannelRead;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerVirtualChannelWrite VirtualChannelWrite;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerVirtualChannelGetData VirtualChannelGetData;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerVirtualChannelSetData VirtualChannelSetData;
+		ALIGN64 psPeerVirtualChannelOpen VirtualChannelOpen;
+		ALIGN64 psPeerVirtualChannelClose VirtualChannelClose;
+		ALIGN64 psPeerVirtualChannelRead VirtualChannelRead;
+		ALIGN64 psPeerVirtualChannelWrite VirtualChannelWrite;
+		ALIGN64 psPeerVirtualChannelGetData VirtualChannelGetData;
+		ALIGN64 psPeerVirtualChannelSetData VirtualChannelSetData;
 
 		ALIGN64 int pId;
 		ALIGN64 UINT32 ack_frame_id;
@@ -182,21 +168,21 @@ extern "C"
 		ALIGN64 BOOL authenticated;
 		ALIGN64 SEC_WINNT_AUTH_IDENTITY identity;
 
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerIsWriteBlocked IsWriteBlocked;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerDrainOutputBuffer DrainOutputBuffer;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerHasMoreToRead HasMoreToRead;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerGetEventHandles GetEventHandles;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerAdjustMonitorsLayout AdjustMonitorsLayout;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerClientCapabilities ClientCapabilities;
+		ALIGN64 psPeerIsWriteBlocked IsWriteBlocked;
+		ALIGN64 psPeerDrainOutputBuffer DrainOutputBuffer;
+		ALIGN64 psPeerHasMoreToRead HasMoreToRead;
+		ALIGN64 psPeerGetEventHandles GetEventHandles;
+		ALIGN64 psPeerAdjustMonitorsLayout AdjustMonitorsLayout;
+		ALIGN64 psPeerClientCapabilities ClientCapabilities;
 #if defined(WITH_FREERDP_DEPRECATED)
 		WINPR_DEPRECATED_VAR("Use freerdp_peer::SspiNtlmHashCallback instead",
-		                     WINPR_ATTR_NODISCARD ALIGN64 psPeerComputeNtlmHash ComputeNtlmHash;)
+		                     ALIGN64 psPeerComputeNtlmHash ComputeNtlmHash;)
 #else
 	UINT64 reserved2;
 #endif
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerLicenseCallback LicenseCallback;
+		ALIGN64 psPeerLicenseCallback LicenseCallback;
 
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerSendChannelPacket SendChannelPacket;
+		ALIGN64 psPeerSendChannelPacket SendChannelPacket;
 
 		/**
 		 * @brief SetState Function pointer allowing to manually set the state of the
@@ -208,38 +194,30 @@ extern "C"
 		 *
 		 * \note Must be called after \b Initialize as that also modifies the state.
 		 */
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerSetState SetState;
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerReachedState ReachedState;
-		WINPR_ATTR_NODISCARD ALIGN64 psSspiNtlmHashCallback SspiNtlmHashCallback;
+		ALIGN64 psPeerSetState SetState;
+		ALIGN64 psPeerReachedState ReachedState;
+		ALIGN64 psSspiNtlmHashCallback SspiNtlmHashCallback;
 		/**
 		 * @brief RemoteCredentials Function pointer that will be called when remote
 		 * credentials guard are used by the peer and we receive the logonCreds (kerberos)
 		 * and supplementary creds (NTLM).
 		 */
-		WINPR_ATTR_NODISCARD ALIGN64 psPeerRemoteCredentials RemoteCredentials;
+		ALIGN64 psPeerRemoteCredentials RemoteCredentials;
 	};
 
 	FREERDP_API void freerdp_peer_context_free(freerdp_peer* client);
 
-	WINPR_ATTR_NODISCARD
 	FREERDP_API BOOL freerdp_peer_context_new(freerdp_peer* client);
-
-	WINPR_ATTR_NODISCARD
 	FREERDP_API BOOL freerdp_peer_context_new_ex(freerdp_peer* client, const rdpSettings* settings);
 
-	WINPR_ATTR_NODISCARD
 	FREERDP_API const char* freerdp_peer_os_major_type_string(freerdp_peer* client);
-
-	WINPR_ATTR_NODISCARD
 	FREERDP_API const char* freerdp_peer_os_minor_type_string(freerdp_peer* client);
 
 	FREERDP_API void freerdp_peer_free(freerdp_peer* client);
 
 	WINPR_ATTR_MALLOC(freerdp_peer_free, 1)
-	WINPR_ATTR_NODISCARD
 	FREERDP_API freerdp_peer* freerdp_peer_new(int sockfd);
 
-	WINPR_ATTR_NODISCARD
 	FREERDP_API BOOL freerdp_peer_set_local_and_hostname(freerdp_peer* client,
 	                                                     const struct sockaddr_storage* peer_addr);
 
