@@ -529,9 +529,8 @@ BOOL freerdp_channels_data(freerdp* instance, UINT16 channelId, const BYTE* cdat
 	data.pcb = cdata;
 	if (!instance || !data.pcb)
 	{
-		WLog_ERR(TAG, "(%p, %" PRIu16 ", %p, 0x%08x): Invalid arguments",
-		         WINPR_CXX_COMPAT_CAST(const void*, instance), channelId,
-		         WINPR_CXX_COMPAT_CAST(const void*, data.pcb), flags);
+		WLog_ERR(TAG, "(%p, %" PRIu16 ", %p, 0x%08x): Invalid arguments", instance, channelId,
+		         data.pcb, flags);
 		return FALSE;
 	}
 
@@ -803,8 +802,7 @@ BOOL freerdp_client_channel_register(rdpChannels* channels, HANDLE handle,
 	if (!channels || (handle == INVALID_HANDLE_VALUE) || !fkt)
 	{
 		WLog_ERR(TAG, "Invalid function arguments (channels=%p, handle=%p, fkt=%p, userdata=%p",
-		         WINPR_CXX_COMPAT_CAST(const void*, channels), handle,
-		         WINPR_CXX_COMPAT_CAST(const void*, fkt), userdata);
+		         channels, handle, fkt, userdata);
 		return FALSE;
 	}
 
@@ -816,8 +814,7 @@ BOOL freerdp_client_channel_unregister(rdpChannels* channels, HANDLE handle)
 {
 	if (!channels || (handle == INVALID_HANDLE_VALUE))
 	{
-		WLog_ERR(TAG, "Invalid function arguments (channels=%p, handle=%p",
-		         WINPR_CXX_COMPAT_CAST(const void*, channels), handle);
+		WLog_ERR(TAG, "Invalid function arguments (channels=%p, handle=%p", channels, handle);
 		return FALSE;
 	}
 
@@ -864,7 +861,7 @@ UINT freerdp_channels_disconnect(rdpChannels* channels, freerdp* instance)
 	if (!channels->connected)
 		return 0;
 
-	freerdp_channels_check_fds(channels, instance);
+	(void)freerdp_channels_check_fds(channels, instance);
 
 	/* tell all libraries we are shutting down */
 	for (int index = 0; index < channels->clientDataCount; index++)
@@ -894,7 +891,7 @@ UINT freerdp_channels_disconnect(rdpChannels* channels, freerdp* instance)
 	channels->connected = FALSE;
 
 	/* Flush pending messages */
-	freerdp_channels_check_fds(channels, instance);
+	(void)freerdp_channels_check_fds(channels, instance);
 	return error;
 }
 
@@ -907,7 +904,7 @@ void freerdp_channels_close(rdpChannels* channels, freerdp* instance)
 	WINPR_ASSERT(instance);
 
 	MessageQueue_PostQuit(channels->queue, 0);
-	freerdp_channels_check_fds(channels, instance);
+	(void)freerdp_channels_check_fds(channels, instance);
 
 	/* tell all libraries we are shutting down */
 	for (int index = 0; index < channels->clientDataCount; index++)
