@@ -54,6 +54,8 @@
 
 #define TAG FREERDP_TAG("core.nla")
 
+// #define SERVER_KEY "Software\\" FREERDP_VENDOR_STRING "\\" FREERDP_PRODUCT_STRING "\\Server"
+
 #define NLA_AUTH_PKG NEGO_SSP_NAME
 
 typedef enum
@@ -1248,7 +1250,6 @@ static BOOL nla_credentialTypeValid(UINT32 type)
 }
 
 WINPR_ATTR_MALLOC(free, 1)
-WINPR_ATTR_NODISCARD
 static MSV1_0_REMOTE_SUPPLEMENTAL_CREDENTIAL* nla_read_NtlmCreds(WINPR_ATTR_UNUSED rdpNla* nla,
                                                                  wStream* s)
 {
@@ -1514,7 +1515,7 @@ static BOOL nla_read_ts_credentials(rdpNla* nla, SecBuffer* data)
 			break;
 		}
 		default:
-			WLog_DBG(TAG, "TSCredentials type %d not supported for now", credType);
+			WLog_DBG(TAG, "TSCredentials type " PRIu32 " not supported for now", credType);
 			ret = FALSE;
 			break;
 	}
@@ -2025,7 +2026,7 @@ BOOL nla_send(rdpNla* nla)
 	if (nla->errorCode && nla->peerVersion >= 3 && nla->peerVersion != 5)
 	{
 		WLog_DBG(TAG, "   ----->> error code %s 0x%08" PRIx32, NtStatus2Tag(nla->errorCode),
-		         WINPR_CXX_COMPAT_CAST(uint32_t, nla->errorCode));
+		         nla->errorCode);
 		if (!WinPrAsn1EncContextualInteger(
 		        enc, 4, WINPR_ASSERTING_INT_CAST(WinPrAsn1_INTEGER, nla->errorCode)))
 			goto fail;
@@ -2152,7 +2153,7 @@ static int nla_decode_ts_request(rdpNla* nla, wStream* s)
 					return -1;
 				nla->errorCode = val;
 				WLog_DBG(TAG, "   <<----- error code %s 0x%08" PRIx32, NtStatus2Tag(nla->errorCode),
-				         WINPR_CXX_COMPAT_CAST(uint32_t, nla->errorCode));
+				         nla->errorCode);
 				break;
 			case 5:
 				WLog_DBG(TAG, "   <<----- client nonce");
@@ -2242,9 +2243,8 @@ int nla_recv_pdu(rdpNla* nla, wStream* s)
 					break;
 
 				default:
-					WLog_ERR(TAG, "SPNEGO failed with NTSTATUS: %s [0x%08" PRIx32 "]",
-					         NtStatus2Tag(nla->errorCode),
-					         WINPR_CXX_COMPAT_CAST(uint32_t, nla->errorCode));
+					WLog_ERR(TAG, "SPNEGO failed with NTSTATUS: %s [0x%08" PRIX32 "]",
+					         NtStatus2Tag(nla->errorCode), nla->errorCode);
 					code = FREERDP_ERROR_AUTHENTICATION_FAILED;
 					break;
 			}
@@ -2454,14 +2454,3 @@ SECURITY_STATUS nla_QueryContextAttributes(rdpNla* nla, DWORD ulAttr, PVOID pBuf
 
 	return table->QueryContextAttributes(&context, ulAttr, pBuffer);
 }
-
-SECURITY_STATUS nla_FreeContextBuffer(rdpNla* nla, PVOID pBuffer)
-{
-	WINPR_ASSERT(nla);
-
-	SecurityFunctionTable* table = NULL;
-	CtxtHandle context = { 0 };
-	credssp_auth_tableAndContext(nla->auth, &table, &context);
-
-	return table->FreeContextBuffer(pBuffer);
-}
