@@ -435,7 +435,8 @@ UINT32 gdi_get_pixel_format(UINT32 bitsPerPixel)
 
 gdiBitmap* gdi_bitmap_new_ex(rdpGdi* gdi, int width, int height, int bpp, BYTE* data)
 {
-	gdiBitmap* bitmap = (gdiBitmap*)calloc(1, sizeof(gdiBitmap));
+	gdiBitmap* bitmap = NULL;
+	bitmap = (gdiBitmap*)calloc(1, sizeof(gdiBitmap));
 
 	if (!bitmap)
 		goto fail_bitmap;
@@ -487,10 +488,8 @@ BOOL gdi_bitmap_update(rdpContext* context, const BITMAP_UPDATE* bitmapUpdate)
 		WLog_ERR(TAG,
 		         "Invalid arguments: context=%p, bitmapUpdate=%p, context->gdi=%p, "
 		         "context->codecs=%p",
-		         WINPR_CXX_COMPAT_CAST(const void*, context),
-		         WINPR_CXX_COMPAT_CAST(const void*, bitmapUpdate),
-		         WINPR_CXX_COMPAT_CAST(const void*, context ? context->gdi : NULL),
-		         WINPR_CXX_COMPAT_CAST(const void*, context ? context->codecs : NULL));
+		         context, bitmapUpdate, context ? context->gdi : NULL,
+		         context ? context->codecs : NULL);
 		return FALSE;
 	}
 
@@ -536,13 +535,12 @@ BOOL gdi_bitmap_update(rdpContext* context, const BITMAP_UPDATE* bitmapUpdate)
 
 static BOOL gdi_palette_update(rdpContext* context, const PALETTE_UPDATE* palette)
 {
+	rdpGdi* gdi = NULL;
+
 	if (!context || !palette)
 		return FALSE;
 
-	rdpGdi* gdi = context->gdi;
-	if (!gdi)
-		return FALSE;
-
+	gdi = context->gdi;
 	gdi->palette.format = gdi->dstFormat;
 
 	for (UINT32 index = 0; index < palette->number; index++)
@@ -557,12 +555,12 @@ static BOOL gdi_palette_update(rdpContext* context, const PALETTE_UPDATE* palett
 
 static BOOL gdi_set_bounds(rdpContext* context, const rdpBounds* bounds)
 {
+	rdpGdi* gdi = NULL;
+
 	if (!context)
 		return FALSE;
 
-	rdpGdi* gdi = context->gdi;
-	if (!gdi || !gdi->drawing)
-		return FALSE;
+	gdi = context->gdi;
 
 	if (bounds)
 	{
@@ -577,12 +575,12 @@ static BOOL gdi_set_bounds(rdpContext* context, const rdpBounds* bounds)
 
 static BOOL gdi_dstblt(rdpContext* context, const DSTBLT_ORDER* dstblt)
 {
+	rdpGdi* gdi = NULL;
+
 	if (!context || !dstblt)
 		return FALSE;
 
-	rdpGdi* gdi = context->gdi;
-	if (!gdi || !gdi->drawing)
-		return FALSE;
+	gdi = context->gdi;
 	return gdi_BitBlt(gdi->drawing->hdc, dstblt->nLeftRect, dstblt->nTopRect, dstblt->nWidth,
 	                  dstblt->nHeight, NULL, 0, 0, gdi_rop3_code_checked(dstblt->bRop),
 	                  &gdi->palette);
@@ -590,9 +588,6 @@ static BOOL gdi_dstblt(rdpContext* context, const DSTBLT_ORDER* dstblt)
 
 static BOOL gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt)
 {
-	WINPR_ASSERT(context);
-	WINPR_ASSERT(patblt);
-
 	const rdpBrush* brush = &patblt->brush;
 	UINT32 foreColor = 0;
 	UINT32 backColor = 0;
@@ -607,9 +602,6 @@ static BOOL gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt)
 	BYTE data[8 * 8 * 4];
 	HGDI_BITMAP hBmp = NULL;
 
-	if (!gdi || !gdi->drawing || !gdi->drawing->hdc)
-		return FALSE;
-
 	if (!gdi_decode_color(gdi, patblt->foreColor, &foreColor, NULL))
 		return FALSE;
 
@@ -703,13 +695,12 @@ out_error:
 
 static BOOL gdi_scrblt(rdpContext* context, const SCRBLT_ORDER* scrblt)
 {
+	rdpGdi* gdi = NULL;
+
 	if (!context || !context->gdi)
 		return FALSE;
 
-	rdpGdi* gdi = context->gdi;
-	if (!gdi->drawing || !gdi->primary)
-		return FALSE;
-
+	gdi = context->gdi;
 	return gdi_BitBlt(gdi->drawing->hdc, scrblt->nLeftRect, scrblt->nTopRect, scrblt->nWidth,
 	                  scrblt->nHeight, gdi->primary->hdc, scrblt->nXSrc, scrblt->nYSrc,
 	                  gdi_rop3_code_checked(scrblt->bRop), &gdi->palette);
@@ -717,9 +708,6 @@ static BOOL gdi_scrblt(rdpContext* context, const SCRBLT_ORDER* scrblt)
 
 static BOOL gdi_opaque_rect(rdpContext* context, const OPAQUE_RECT_ORDER* opaque_rect)
 {
-	WINPR_ASSERT(context);
-	WINPR_ASSERT(opaque_rect);
-
 	GDI_RECT rect;
 	HGDI_BRUSH hBrush = NULL;
 	UINT32 brush_color = 0;
@@ -729,11 +717,7 @@ static BOOL gdi_opaque_rect(rdpContext* context, const OPAQUE_RECT_ORDER* opaque
 	INT32 y = opaque_rect->nTopRect;
 	INT32 w = opaque_rect->nWidth;
 	INT32 h = opaque_rect->nHeight;
-	if (!gdi || !gdi->drawing)
-		return FALSE;
-
-	if (!gdi_CRgnToRect(x, y, w, h, &rect))
-		return TRUE;
+	gdi_CRgnToRect(x, y, w, h, &rect);
 
 	if (!gdi_decode_color(gdi, opaque_rect->color, &brush_color, NULL))
 		return FALSE;
@@ -749,21 +733,16 @@ static BOOL gdi_opaque_rect(rdpContext* context, const OPAQUE_RECT_ORDER* opaque
 static BOOL gdi_multi_opaque_rect(rdpContext* context,
                                   const MULTI_OPAQUE_RECT_ORDER* multi_opaque_rect)
 {
-	WINPR_ASSERT(context);
-	WINPR_ASSERT(multi_opaque_rect);
-
-	GDI_RECT rect = { 0 };
+	GDI_RECT rect;
+	HGDI_BRUSH hBrush = NULL;
 	UINT32 brush_color = 0;
 	rdpGdi* gdi = context->gdi;
 	BOOL ret = TRUE;
 
-	if (!gdi || !gdi->drawing)
-		return FALSE;
-
 	if (!gdi_decode_color(gdi, multi_opaque_rect->color, &brush_color, NULL))
 		return FALSE;
 
-	HGDI_BRUSH hBrush = gdi_CreateSolidBrush(brush_color);
+	hBrush = gdi_CreateSolidBrush(brush_color);
 
 	if (!hBrush)
 		return FALSE;
@@ -776,9 +755,7 @@ static BOOL gdi_multi_opaque_rect(rdpContext* context,
 		const INT32 w = rectangle->width;
 		const INT32 h = rectangle->height;
 
-		ret = FALSE;
-		if (!gdi_CRgnToRect(x, y, w, h, &rect))
-			break;
+		gdi_CRgnToRect(x, y, w, h, &rect);
 		ret = gdi_FillRect(gdi->drawing->hdc, &rect, hBrush);
 
 		if (!ret)
@@ -796,8 +773,7 @@ static BOOL gdi_line_to(rdpContext* context, const LINE_TO_ORDER* lineTo)
 	WINPR_ASSERT(lineTo);
 
 	rdpGdi* gdi = context->gdi;
-	if (!gdi || !gdi->drawing || !gdi->drawing->hdc)
-		return FALSE;
+	WINPR_ASSERT(gdi);
 
 	if (!gdi_decode_color(gdi, lineTo->penColor, &color, NULL))
 		return FALSE;
@@ -823,8 +799,7 @@ static BOOL gdi_polyline(rdpContext* context, const POLYLINE_ORDER* polyline)
 	WINPR_ASSERT(polyline);
 
 	rdpGdi* gdi = context->gdi;
-	if (!gdi || !gdi->drawing || !gdi->drawing->hdc)
-		return FALSE;
+	WINPR_ASSERT(gdi);
 
 	UINT32 color = 0;
 	if (!gdi_decode_color(gdi, polyline->penColor, &color, NULL))
@@ -858,13 +833,14 @@ static BOOL gdi_polyline(rdpContext* context, const POLYLINE_ORDER* polyline)
 
 static BOOL gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt)
 {
-	if (!context || !memblt || !memblt->bitmap)
+	gdiBitmap* bitmap = NULL;
+	rdpGdi* gdi = NULL;
+
+	if (!context || !memblt || !context->gdi || !memblt->bitmap)
 		return FALSE;
 
-	gdiBitmap* bitmap = (gdiBitmap*)memblt->bitmap;
-	rdpGdi* gdi = context->gdi;
-	if (!gdi || !gdi->drawing)
-		return FALSE;
+	bitmap = (gdiBitmap*)memblt->bitmap;
+	gdi = context->gdi;
 	return gdi_BitBlt(gdi->drawing->hdc, memblt->nLeftRect, memblt->nTopRect, memblt->nWidth,
 	                  memblt->nHeight, bitmap->hdc, memblt->nXSrc, memblt->nYSrc,
 	                  gdi_rop3_code_checked(memblt->bRop), &gdi->palette);
@@ -872,14 +848,8 @@ static BOOL gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt)
 
 static BOOL gdi_mem3blt(rdpContext* context, MEM3BLT_ORDER* mem3blt)
 {
-	WINPR_ASSERT(context);
-	WINPR_ASSERT(mem3blt);
-
 	HGDI_BRUSH originalBrush = NULL;
 	rdpGdi* gdi = context->gdi;
-	if (!gdi || !gdi->drawing)
-		return FALSE;
-
 	BOOL ret = TRUE;
 	const rdpBrush* brush = &mem3blt->brush;
 	gdiBitmap* bitmap = (gdiBitmap*)mem3blt->bitmap;
@@ -1035,11 +1005,6 @@ static BOOL gdi_frame_marker(WINPR_ATTR_UNUSED rdpContext* context,
 static BOOL gdi_surface_frame_marker(rdpContext* context,
                                      const SURFACE_FRAME_MARKER* surfaceFrameMarker)
 {
-	WINPR_ASSERT(context);
-	WINPR_ASSERT(context->gdi);
-	WINPR_ASSERT(context->update);
-	WINPR_ASSERT(surfaceFrameMarker);
-
 	WLog_Print(context->gdi->log, WLOG_DEBUG, "frameId %" PRIu32 " frameAction %" PRIu32 "",
 	           surfaceFrameMarker->frameId, surfaceFrameMarker->frameAction);
 
@@ -1051,9 +1016,8 @@ static BOOL gdi_surface_frame_marker(rdpContext* context,
 		case SURFACECMD_FRAMEACTION_END:
 			if (freerdp_settings_get_uint32(context->settings, FreeRDP_FrameAcknowledge) > 0)
 			{
-				if (!IFCALLRESULT(TRUE, context->update->SurfaceFrameAcknowledge, context,
-				                  surfaceFrameMarker->frameId))
-					return FALSE;
+				IFCALL(context->update->SurfaceFrameAcknowledge, context,
+				       surfaceFrameMarker->frameId);
 			}
 
 			break;
@@ -1066,10 +1030,6 @@ static BOOL gdi_surface_frame_marker(rdpContext* context,
 
 static BOOL intersect_rect(const rdpGdi* gdi, const SURFACE_BITS_COMMAND* cmd, RECTANGLE_16* prect)
 {
-	WINPR_ASSERT(gdi);
-	WINPR_ASSERT(cmd);
-	WINPR_ASSERT(prect);
-
 	const UINT32 w = (const UINT32)gdi->width;
 	const UINT32 h = (const UINT32)gdi->height;
 
@@ -1102,6 +1062,7 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 {
 	BOOL result = FALSE;
 	DWORD format = 0;
+	rdpGdi* gdi = NULL;
 	size_t size = 0;
 	REGION16 region;
 	RECTANGLE_16 cmdRect = { 0 };
@@ -1109,10 +1070,7 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 	if (!context || !cmd)
 		return FALSE;
 
-	rdpGdi* gdi = context->gdi;
-	if (!gdi)
-		return FALSE;
-
+	gdi = context->gdi;
 	WLog_Print(gdi->log, WLOG_DEBUG,
 	           "destLeft %" PRIu32 " destTop %" PRIu32 " destRight %" PRIu32 " destBottom %" PRIu32
 	           " "
@@ -1147,16 +1105,14 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 			if (!nsc_process_message(
 			        context->codecs->nsc, cmd->bmp.bpp, cmd->bmp.width, cmd->bmp.height,
 			        cmd->bmp.bitmapData, cmd->bmp.bitmapDataLength, gdi->primary_buffer, format,
-			        gdi->stride, cmdRect.left, cmdRect.top,
-			        WINPR_ASSERTING_INT_CAST(UINT32, gdi->width),
-			        WINPR_ASSERTING_INT_CAST(UINT32, gdi->height), FREERDP_FLIP_VERTICAL))
+			        gdi->stride, cmdRect.left, cmdRect.top, cmdRect.right - cmdRect.left,
+			        cmdRect.bottom - cmdRect.top, FREERDP_FLIP_VERTICAL))
 			{
 				WLog_ERR(TAG, "Failed to process NSCodec message");
 				goto out;
 			}
 
-			if (!region16_union_rect(&region, &region, &cmdRect))
-				goto out;
+			region16_union_rect(&region, &region, &cmdRect);
 			break;
 
 		case RDP_CODEC_ID_NONE:
@@ -1178,8 +1134,7 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 				goto out;
 			}
 
-			if (!region16_union_rect(&region, &region, &cmdRect))
-				goto out;
+			region16_union_rect(&region, &region, &cmdRect);
 			break;
 
 		default:
@@ -1232,13 +1187,16 @@ out:
 
 static void gdi_register_update_callbacks(rdpUpdate* update)
 {
+	rdpPrimaryUpdate* primary = NULL;
+	const rdpSettings* settings = NULL;
+
 	WINPR_ASSERT(update);
 	WINPR_ASSERT(update->context);
 
-	const rdpSettings* settings = update->context->settings;
+	settings = update->context->settings;
 	WINPR_ASSERT(settings);
 
-	rdpPrimaryUpdate* primary = update->primary;
+	primary = update->primary;
 	WINPR_ASSERT(primary);
 
 	if (freerdp_settings_get_bool(settings, FreeRDP_DeactivateClientDecoding))
