@@ -270,17 +270,20 @@ static BOOL gdi_Bitmap_SetSurface(rdpContext* context, rdpBitmap* bitmap, BOOL p
 /* Glyph Class */
 static BOOL gdi_Glyph_New(rdpContext* context, rdpGlyph* glyph)
 {
+	BYTE* data = NULL;
+	gdiGlyph* gdi_glyph = NULL;
+
 	if (!context || !glyph)
 		return FALSE;
 
-	gdiGlyph* gdi_glyph = (gdiGlyph*)glyph;
+	gdi_glyph = (gdiGlyph*)glyph;
 	gdi_glyph->hdc = gdi_GetDC();
 
 	if (!gdi_glyph->hdc)
 		return FALSE;
 
 	gdi_glyph->hdc->format = PIXEL_FORMAT_MONO;
-	BYTE* data = freerdp_glyph_convert_ex(glyph->cx, glyph->cy, glyph->aj, glyph->cb);
+	data = freerdp_glyph_convert(glyph->cx, glyph->cy, glyph->aj);
 
 	if (!data)
 	{
@@ -354,10 +357,8 @@ static BOOL gdi_Glyph_Draw(rdpContext* context, const rdpGlyph* glyph, INT32 x,
 			if (!brush)
 				return FALSE;
 
-			const BOOL res = gdi_FillRect(gdi->drawing->hdc, &rect, brush);
+			gdi_FillRect(gdi->drawing->hdc, &rect, brush);
 			gdi_DeleteObject((HGDIOBJECT)brush);
-			if (!res)
-				return res;
 		}
 	}
 
@@ -376,10 +377,12 @@ static BOOL gdi_Glyph_Draw(rdpContext* context, const rdpGlyph* glyph, INT32 x,
 static BOOL gdi_Glyph_BeginDraw(rdpContext* context, INT32 x, INT32 y, INT32 width, INT32 height,
                                 UINT32 bgcolor, UINT32 fgcolor, BOOL fOpRedundant)
 {
+	rdpGdi* gdi = NULL;
+
 	if (!context || !context->gdi)
 		return FALSE;
 
-	rdpGdi* gdi = context->gdi;
+	gdi = context->gdi;
 
 	if (!gdi->drawing || !gdi->drawing->hdc)
 		return FALSE;
@@ -392,12 +395,11 @@ static BOOL gdi_Glyph_BeginDraw(rdpContext* context, INT32 x, INT32 y, INT32 wid
 		if (!gdi_decode_color(gdi, fgcolor, &fgcolor, NULL))
 			return FALSE;
 
-		if (!gdi_SetClipRgn(gdi->drawing->hdc, x, y, width, height))
-			return FALSE;
-
+		gdi_SetClipRgn(gdi->drawing->hdc, x, y, width, height);
 		gdi_SetTextColor(gdi->drawing->hdc, bgcolor);
 		gdi_SetBkColor(gdi->drawing->hdc, fgcolor);
 
+		if (1)
 		{
 			GDI_RECT rect = { 0 };
 			HGDI_BRUSH brush = gdi_CreateSolidBrush(fgcolor);
@@ -414,13 +416,10 @@ static BOOL gdi_Glyph_BeginDraw(rdpContext* context, INT32 x, INT32 y, INT32 wid
 			rect.right = x + width - 1;
 			rect.bottom = y + height - 1;
 
-			BOOL res = TRUE;
 			if ((x + width > rect.left) && (y + height > rect.top))
-				res = gdi_FillRect(gdi->drawing->hdc, &rect, brush);
+				gdi_FillRect(gdi->drawing->hdc, &rect, brush);
 
 			gdi_DeleteObject((HGDIOBJECT)brush);
-			if (!res)
-				return FALSE;
 		}
 
 		return gdi_SetNullClipRgn(gdi->drawing->hdc);
