@@ -109,7 +109,6 @@ static void test_peer_context_free(freerdp_peer* client, rdpContext* ctx)
 	}
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL test_peer_context_new(freerdp_peer* client, rdpContext* ctx)
 {
 	testPeerContext* context = (testPeerContext*)ctx;
@@ -131,8 +130,7 @@ static BOOL test_peer_context_new(freerdp_peer* client, rdpContext* ctx)
 
 	{
 		const UINT32 rlgr = freerdp_settings_get_uint32(ctx->settings, FreeRDP_RemoteFxRlgrMode);
-		if (!rfx_context_set_mode(context->rfx_context, WINPR_ASSERTING_INT_CAST(RLGR_MODE, rlgr)))
-			goto fail;
+		rfx_context_set_mode(context->rfx_context, WINPR_ASSERTING_INT_CAST(RLGR_MODE, rlgr));
 	}
 
 	if (!(context->nsc_context = nsc_context_new()))
@@ -154,7 +152,6 @@ fail:
 	return FALSE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL test_peer_init(freerdp_peer* client)
 {
 	WINPR_ASSERT(client);
@@ -165,7 +162,6 @@ static BOOL test_peer_init(freerdp_peer* client)
 	return freerdp_peer_context_new(client);
 }
 
-WINPR_ATTR_NODISCARD
 static wStream* test_peer_stream_init(testPeerContext* context)
 {
 	WINPR_ASSERT(context);
@@ -194,8 +190,7 @@ static void test_peer_begin_frame(freerdp_peer* client)
 	fm.frameAction = SURFACECMD_FRAMEACTION_BEGIN;
 	fm.frameId = context->frame_id;
 	WINPR_ASSERT(update->SurfaceFrameMarker);
-	if (!update->SurfaceFrameMarker(update->context, &fm))
-		WLog_WARN(TAG, "SurfaceFrameMarker failed");
+	update->SurfaceFrameMarker(update->context, &fm);
 }
 
 static void test_peer_end_frame(freerdp_peer* client)
@@ -215,12 +210,10 @@ static void test_peer_end_frame(freerdp_peer* client)
 	fm.frameAction = SURFACECMD_FRAMEACTION_END;
 	fm.frameId = context->frame_id;
 	WINPR_ASSERT(update->SurfaceFrameMarker);
-	if (!update->SurfaceFrameMarker(update->context, &fm))
-		WLog_WARN(TAG, "SurfaceFrameMarker failed");
+	update->SurfaceFrameMarker(update->context, &fm);
 	context->frame_id++;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL stream_surface_bits_supported(const rdpSettings* settings)
 {
 	const UINT32 supported =
@@ -290,13 +283,11 @@ static BOOL test_peer_draw_background(freerdp_peer* client, const RFX_RECT* rect
 	else
 	{
 		WLog_DBG(TAG, "Using NSCodec");
-		if (!nsc_context_set_parameters(context->nsc_context, NSC_COLOR_FORMAT, colorFormat))
-			goto out;
+		nsc_context_set_parameters(context->nsc_context, NSC_COLOR_FORMAT, colorFormat);
 
 		WINPR_ASSERT(bpp <= UINT16_MAX);
-		if (!nsc_compose_message(context->nsc_context, s, rgb_data, rect->width, rect->height,
-		                         (UINT32)(bpp * rect->width)))
-			goto out;
+		nsc_compose_message(context->nsc_context, s, rgb_data, rect->width, rect->height,
+		                    (UINT32)(bpp * rect->width));
 		const UINT32 NSCodecId = freerdp_settings_get_uint32(settings, FreeRDP_NSCodecId);
 		WINPR_ASSERT(NSCodecId <= UINT16_MAX);
 		cmd.bmp.codecID = (UINT16)NSCodecId;
@@ -315,13 +306,13 @@ static BOOL test_peer_draw_background(freerdp_peer* client, const RFX_RECT* rect
 	cmd.bmp.bitmapDataLength = (UINT32)Stream_GetPosition(s);
 	cmd.bmp.bitmapData = Stream_Buffer(s);
 
-	ret = update->SurfaceBits(update->context, &cmd);
+	update->SurfaceBits(update->context, &cmd);
+	ret = TRUE;
 out:
 	free(rgb_data);
 	return ret;
 }
 
-WINPR_ATTR_NODISCARD
 static int open_icon(wImage* img)
 {
 	char* paths[] = { SAMPLE_RESOURCE_ROOT, "." };
@@ -347,7 +338,6 @@ static int open_icon(wImage* img)
 	return -1;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL test_peer_load_icon(freerdp_peer* client)
 {
 	testPeerContext* context = NULL;
@@ -439,17 +429,14 @@ static void test_send_cursor_update(freerdp_peer* client, UINT32 x, UINT32 y)
 		if (RemoteFxCodec)
 		{
 			rfx_context_set_pixel_format(context->rfx_context, colorFormat);
-			if (!rfx_compose_message(context->rfx_context, s, &rect, 1, context->image->data,
-			                         rect.width, rect.height, context->image->scanline))
-				WLog_WARN(TAG, "rfx_compose_message failed");
+			rfx_compose_message(context->rfx_context, s, &rect, 1, context->image->data, rect.width,
+			                    rect.height, context->image->scanline);
 		}
 		else
 		{
-			if (!nsc_context_set_parameters(context->nsc_context, NSC_COLOR_FORMAT, colorFormat))
-				WLog_WARN(TAG, "nsc_context_set_parameters failed");
-			else if (!nsc_compose_message(context->nsc_context, s, context->image->data, rect.width,
-			                              rect.height, context->image->scanline))
-				WLog_WARN(TAG, "rfx_compose_message failed");
+			nsc_context_set_parameters(context->nsc_context, NSC_COLOR_FORMAT, colorFormat);
+			nsc_compose_message(context->nsc_context, s, context->image->data, rect.width,
+			                    rect.height, context->image->scanline);
 		}
 	}
 
@@ -466,10 +453,7 @@ static void test_send_cursor_update(freerdp_peer* client, UINT32 x, UINT32 y)
 	rdpUpdate* update = client->context->update;
 	WINPR_ASSERT(update);
 	WINPR_ASSERT(update->SurfaceBits);
-	if (!update->SurfaceBits(update->context, &cmd))
-	{
-		WLog_WARN(TAG, "update->SurfaceBits failed");
-	}
+	update->SurfaceBits(update->context, &cmd);
 	context->icon_x = x;
 	context->icon_y = y;
 }
@@ -500,7 +484,6 @@ static void test_peer_draw_icon(freerdp_peer* client, UINT32 x, UINT32 y)
 	test_peer_end_frame(client);
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL test_sleep_tsdiff(UINT32* old_sec, UINT32* old_usec, UINT32 new_sec, UINT32 new_usec)
 {
 	INT64 sec = 0;
@@ -582,8 +565,7 @@ static BOOL tf_peer_dump_rfx(freerdp_peer* client)
 			break;
 
 		record.data = Stream_Buffer(s);
-		if (!pcap_get_next_record_content(pcap_rfx, &record))
-			break;
+		pcap_get_next_record_content(pcap_rfx, &record);
 		Stream_SetPosition(s, Stream_Capacity(s));
 
 		if (info->test_dump_rfx_realtime &&
@@ -592,8 +574,7 @@ static BOOL tf_peer_dump_rfx(freerdp_peer* client)
 			break;
 
 		WINPR_ASSERT(update->SurfaceCommand);
-		if (!update->SurfaceCommand(update->context, s))
-			break;
+		update->SurfaceCommand(update->context, s);
 
 		WINPR_ASSERT(client->CheckFileDescriptor);
 		if (client->CheckFileDescriptor(client) != TRUE)
@@ -607,7 +588,6 @@ fail:
 	return rc;
 }
 
-WINPR_ATTR_NODISCARD
 static DWORD WINAPI tf_debug_channel_thread_func(LPVOID arg)
 {
 	void* fd = NULL;
@@ -679,7 +659,6 @@ fail:
 	return 0;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_post_connect(freerdp_peer* client)
 {
 	testPeerContext* context = NULL;
@@ -713,6 +692,7 @@ static BOOL tf_peer_post_connect(freerdp_peer* client)
 		/* A real server may perform OS login here if NLA is not executed previously. */
 	}
 
+	WLog_DBG(TAG, "");
 	WLog_DBG(TAG, "Client requested desktop: %" PRIu32 "x%" PRIu32 "x%" PRIu32 "",
 	         freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
 	         freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
@@ -771,19 +751,16 @@ static BOOL tf_peer_post_connect(freerdp_peer* client)
 
 	if (WTSVirtualChannelManagerIsChannelJoined(context->vcm, RDPSND_CHANNEL_NAME))
 	{
-		if (!sf_peer_rdpsnd_init(context)) /* Audio Output */
-			return FALSE;
+		sf_peer_rdpsnd_init(context); /* Audio Output */
 	}
 
 	if (WTSVirtualChannelManagerIsChannelJoined(context->vcm, ENCOMSP_SVC_CHANNEL_NAME))
 	{
-		if (!sf_peer_encomsp_init(context)) /* Lync Multiparty */
-			return FALSE;
+		sf_peer_encomsp_init(context); /* Lync Multiparty */
 	}
 
 	/* Dynamic Virtual Channels */
-	if (!sf_peer_audin_init(context)) /* Audio Input */
-		return FALSE;
+	sf_peer_audin_init(context); /* Audio Input */
 
 #if defined(CHANNEL_AINPUT_SERVER)
 	sf_peer_ainput_init(context);
@@ -793,16 +770,17 @@ static BOOL tf_peer_post_connect(freerdp_peer* client)
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_activate(freerdp_peer* client)
 {
-	BOOL rc = TRUE;
+	testPeerContext* context = NULL;
+	rdpSettings* settings = NULL;
+
 	WINPR_ASSERT(client);
 
-	testPeerContext* context = (testPeerContext*)client->context;
+	context = (testPeerContext*)client->context;
 	WINPR_ASSERT(context);
 
-	rdpSettings* settings = client->context->settings;
+	settings = client->context->settings;
 	WINPR_ASSERT(settings);
 
 	struct server_info* info = client->ContextExtra;
@@ -832,14 +810,13 @@ static BOOL tf_peer_activate(freerdp_peer* client)
 			.height = (UINT16)freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight)
 		};
 		test_peer_begin_frame(client);
-		rc = test_peer_draw_background(client, &rect);
+		test_peer_draw_background(client, &rect);
 		test_peer_end_frame(client);
 	}
 
-	return rc;
+	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_synchronize_event(rdpInput* input, UINT32 flags)
 {
 	WINPR_UNUSED(input);
@@ -848,7 +825,6 @@ static BOOL tf_peer_synchronize_event(rdpInput* input, UINT32 flags)
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_keyboard_event(rdpInput* input, UINT16 flags, UINT8 code)
 {
 	freerdp_peer* client = NULL;
@@ -902,8 +878,7 @@ static BOOL tf_peer_keyboard_event(rdpInput* input, UINT16 flags, UINT8 code)
 			return FALSE;
 
 		WINPR_ASSERT(update->DesktopResize);
-		if (!update->DesktopResize(update->context))
-			return FALSE;
+		update->DesktopResize(update->context);
 		tcontext->activated = FALSE;
 	}
 	else if (((flags & KBD_FLAGS_RELEASE) == 0) && code == RDP_SCANCODE_KEY_C) /* 'c' key */
@@ -918,8 +893,7 @@ static BOOL tf_peer_keyboard_event(rdpInput* input, UINT16 flags, UINT8 code)
 	else if (((flags & KBD_FLAGS_RELEASE) == 0) && code == RDP_SCANCODE_KEY_X) /* 'x' key */
 	{
 		WINPR_ASSERT(client->Close);
-		if (!client->Close(client))
-			return FALSE;
+		client->Close(client);
 	}
 	else if (((flags & KBD_FLAGS_RELEASE) == 0) && code == RDP_SCANCODE_KEY_R) /* 'r' key */
 	{
@@ -938,7 +912,6 @@ static BOOL tf_peer_keyboard_event(rdpInput* input, UINT16 flags, UINT8 code)
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_unicode_keyboard_event(rdpInput* input, UINT16 flags, UINT16 code)
 {
 	WINPR_UNUSED(input);
@@ -950,7 +923,6 @@ static BOOL tf_peer_unicode_keyboard_event(rdpInput* input, UINT16 flags, UINT16
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_mouse_event(rdpInput* input, UINT16 flags, UINT16 x, UINT16 y)
 {
 	WINPR_UNUSED(flags);
@@ -963,7 +935,6 @@ static BOOL tf_peer_mouse_event(rdpInput* input, UINT16 flags, UINT16 x, UINT16
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static UINT32 add(UINT32 old, UINT32 max, INT16 diff)
 {
 	INT64 val = old;
@@ -975,7 +946,6 @@ static UINT32 add(UINT32 old, UINT32 max, INT16 diff)
 	return WINPR_ASSERTING_INT_CAST(uint32_t, val);
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_rel_mouse_event(rdpInput* input, UINT16 flags, INT16 xDelta, INT16 yDelta)
 {
 	WINPR_UNUSED(flags);
@@ -999,7 +969,6 @@ static BOOL tf_peer_rel_mouse_event(rdpInput* input, UINT16 flags, INT16 xDelta,
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_extended_mouse_event(rdpInput* input, UINT16 flags, UINT16 x, UINT16 y)
 {
 	WINPR_UNUSED(flags);
@@ -1014,7 +983,6 @@ static BOOL tf_peer_extended_mouse_event(rdpInput* input, UINT16 flags, UINT16 x
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_refresh_rect(rdpContext* context, BYTE count, const RECTANGLE_16* areas)
 {
 	WINPR_UNUSED(context);
@@ -1032,7 +1000,6 @@ static BOOL tf_peer_refresh_rect(rdpContext* context, BYTE count, const RECTANGL
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL tf_peer_suppress_output(rdpContext* context, BYTE allow, const RECTANGLE_16* area)
 {
 	WINPR_UNUSED(context);
@@ -1052,7 +1019,6 @@ static BOOL tf_peer_suppress_output(rdpContext* context, BYTE allow, const RECTA
 	return TRUE;
 }
 
-WINPR_ATTR_NODISCARD
 static int hook_peer_write_pdu(rdpTransport* transport, wStream* s)
 {
 	UINT64 ts = 0;
@@ -1116,7 +1082,6 @@ fail:
 	return -1;
 }
 
-WINPR_ATTR_NODISCARD
 static DWORD WINAPI test_peer_mainloop(LPVOID arg)
 {
 	BOOL rc = 0;
@@ -1232,8 +1197,7 @@ static DWORD WINAPI test_peer_mainloop(LPVOID arg)
 		replay = *cb;
 		context->io = *cb;
 		replay.WritePdu = hook_peer_write_pdu;
-		if (!freerdp_set_io_callbacks(client->context, &replay))
-			goto fail;
+		freerdp_set_io_callbacks(client->context, &replay);
 	}
 
 	WLog_INFO(TAG, "We've got a client %s", client->local ? "(local)" : client->hostname);
@@ -1322,7 +1286,6 @@ fail:
 	return error;
 }
 
-WINPR_ATTR_NODISCARD
 static BOOL test_peer_accepted(freerdp_listener* instance, freerdp_peer* client)
 {
 	HANDLE hThread = NULL;
@@ -1406,7 +1369,6 @@ static void print_entry(FILE* fp, WINPR_FORMAT_ARG const char* fmt, const char*
 }
 WINPR_PRAGMA_DIAG_POP
 
-WINPR_ATTR_NODISCARD
 static int usage(const char* app, const char* invalid)
 {
 	FILE* fp = stdout;
